Package: mwTensor
Type: Package
Title: Multi-Way Component Analysis
Version: 1.2.2
Authors@R: c(person("Koki", "Tsuyuzaki", role = c("aut", "cre"),
                      email = "k.t.the-answer@hotmail.co.jp"))
Suggests: testthat, knitr, rmarkdown
VignetteBuilder: knitr
Depends: R (>= 4.1.0)
Imports: methods, MASS, rTensor, nnTensor, ccTensor, iTensor, igraph
Description: For single tensor data, any matrix factorization method can be specified the matricised tensor in each dimension by Multi-way Component Analysis (MWCA). An originally extended MWCA is also implemented to specify and decompose multiple matrices and tensors simultaneously (CoupledMWCA). See the reference section of GitHub README.md <https://github.com/rikenbit/mwTensor>, for details of the methods.
License: MIT + file LICENSE
URL: https://github.com/rikenbit/mwTensor
NeedsCompilation: no
Packaged: 2026-05-07 05:24:02 UTC; koki
Author: Koki Tsuyuzaki [aut, cre]
Maintainer: Koki Tsuyuzaki <k.t.the-answer@hotmail.co.jp>
Repository: CRAN
Date/Publication: 2026-05-07 06:01:06 UTC
Built: R 4.6.0; ; 2026-05-07 08:22:32 UTC; unix
