importFrom(methods, is, new)
importFrom(MASS, ginv)
importFrom(rTensor, as.tensor, vec, cs_unfold, khatri_rao_list, ttm)
importFrom(nnTensor, recTensor, NMF, plotTensor3D, plotTensor2D)
importFrom(ccTensor, CX)
importFrom(iTensor, ICA)
importFrom(igraph, graph_from_incidence_matrix, max_bipartite_match)
importFrom(graphics, layout, par, text)
importFrom(grDevices, dev.off, png)
importFrom(stats, cor, rpois, runif)

export(mySVD)
export(myALS_SVD)
export(myNMF)
export(myICA)
export(myCX)
export(MWCA)
export(CoupledMWCA)
export(defaultMWCAParams)
export(defaultCoupledMWCAParams)
export(checkCoupledMWCA)
export(MWCAProgram)
export(MWCAProgramBlock)
export(MWCAProgramFactor)
export(MWCAProgramRefinement)
export(validateMWCAProgram)
export(compileMWCAProgram)
export(executeMWCAProgram)
S3method(print, MWCAProgram)
S3method(summary, MWCAProgram)
export(initCoupledMWCA)
export(refineFactor)
exportClasses(RefinedFactor)
exportClasses(CoupledMWCAInit)
export(plotTensor3Ds)
export(toyModel)
